'use strict';

var normalizeUrl = require('normalize-url');

var srcByModuleId = Object.create(null);
var debounce = require('lodash/debounce');

var noDocument = typeof document === 'undefined';
var forEach = Array.prototype.forEach;

var noop = function noop() {};

var getCurrentScriptUrl = function getCurrentScriptUrl(moduleId) {
    var src = srcByModuleId[moduleId];

    if (!src) {
        if (document.currentScript) {
            src = document.currentScript.src;
        } else {
            var scripts = document.getElementsByTagName('script');
            var lastScriptTag = scripts[scripts.length - 1];

            if (lastScriptTag) {
                src = lastScriptTag.src;
            }
        }
        srcByModuleId[moduleId] = src;
    }

    return function (fileMap) {
        var splitResult = /([^\\/]+)\.js$/.exec(src);
        var filename = splitResult && splitResult[1];
        if (!filename) {
            return [src.replace('.js', '.css')];
        }
        return fileMap.split(',').map(function (mapRule) {
            var reg = new RegExp(filename + '\\.js$', 'g');
            return normalizeUrl(src.replace(reg, mapRule.replace(/{fileName}/g, filename) + '.css'), { stripWWW: false });
        });
    };
};

function updateCss(el, url) {
    if (!url) {
        url = el.href.split('?')[0];
    }
    if (el.isLoaded === false) {
        // We seem to be about to replace a css link that hasn't loaded yet.
        // We're probably changing the same file more than once.
        return;
    }
    if (!url || !(url.indexOf('.css') > -1)) return;

    el.visited = true;
    var newEl = el.cloneNode();

    newEl.isLoaded = false;

    newEl.addEventListener('load', function () {
        newEl.isLoaded = true;
        el.parentNode.removeChild(el);
    });

    newEl.addEventListener('error', function () {
        newEl.isLoaded = true;
        el.parentNode.removeChild(el);
    });

    newEl.href = url + '?' + Date.now();
    el.parentNode.appendChild(newEl);
}

function getReloadUrl(href, src) {
    href = normalizeUrl(href, { stripWWW: false });
    var ret = void 0;
    // eslint-disable-next-line array-callback-return
    src.some(function (url) {
        if (href.indexOf(src) > -1) {
            ret = url;
        }
    });
    return ret;
}

function reloadStyle(src) {
    var elements = document.querySelectorAll('link');
    var loaded = false;

    forEach.call(elements, function (el) {
        if (el.visited === true) return;

        var url = getReloadUrl(el.href, src);
        if (url) {
            updateCss(el, url);
            loaded = true;
        }
    });

    return loaded;
}

function reloadAll() {
    var elements = document.querySelectorAll('link');
    forEach.call(elements, function (el) {
        if (el.visited === true) return;
        updateCss(el);
    });
}

module.exports = function (moduleId, options) {
    if (noDocument) {
        return noop;
    }

    var getScriptSrc = getCurrentScriptUrl(moduleId);

    function update() {
        var src = getScriptSrc(options.fileMap);
        var reloaded = reloadStyle(src);
        if (reloaded && !options.reloadAll) {
            console.log('[HMR] css reload %s', src.join(' '));
        } else {
            console.log('[HMR] Reload all css');
            reloadAll();
        }
    }

    return debounce(update, 10);
};